/* REXX file to add commit heuristics settings to HPFS386.INI. */

"@echo off"
CALL RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
CALL SysLoadFuncs

/* Get the NLS versions of "Yes" and "No". */

yesmsg = TRANSLATE(SysGetMessage(4300, "NET.MSG"))
nomsg  = TRANSLATE(SysGetMessage(4301, "NET.MSG"))

/* Read in parameters.  "YES" or "NO" is required.  A drive letter for the    */
/* boot drive is optional.                                                    */

PARSE UPPER ARG PerfSet bootdrive .
IF ((PerfSet <> yesmsg & PerfSet <> nomsg) | (bootdrive <> "" & (LENGTH(bootdrive) <> 2 | \DATATYPE(SUBSTR(bootdrive, 1, 1), "M") | SUBSTR(bootdrive, 2, 1) <> ":"))) THEN DO
   SAY SysGetMessage(3953, "NET.MSG")   /* "The syntax is incorrect."         */
   SAY SysGetMessage(4381, "NET.MSG")   /* "The syntax of this command is:"   */
   diskmsg = SysGetMessage(4710, "NET.MSG")     /* Get word for "Disk"        */
   SAY "HPFSPERF" yesmsg "|" nomsg "[" || diskmsg || ":]"
   RETURN
END

/* Find the boot drive, if it wasn't given.  The boot drive is needed to      */
/* locate CONFIG.SYS and HPFS386.INI.  The boot drive is found by looking at  */
/* the PATH and finding the first location of the string ":\OS2\SYSTEM;".     */
/* The drive letter ahead of this string is taken to be the boot drive.       */

IF bootdrive = "" THEN DO
   path = VALUE("PATH",,"OS2ENVIRONMENT")
   bootdrivepos = POS(":\OS2\SYSTEM;", path) - 1
   IF bootdrivepos < 1 THEN DO
      SAY SysGetMessage(10)             /* "The environment is incorrect."    */
      RETURN
   END
   bootdrive = SUBSTR(path, bootdrivepos, 2)
END

/* Check to make sure HPFS386 has been installed on the boot drive.  A simple */
/* check for the existence of d:\IBM386FS\HPFS386.IFS will verify that it is  */
/* installed.                                                                 */

rc = SysFileTree(bootdrive || "\IBM386FS\HPFS386.IFS", file, "F")
IF file.0 = 0 THEN DO
   SAY bootdrive || "\IBM386FS\HPFS386.IFS"
   SAY SysGetMessage(2)                 /* "File not found."                  */
   RETURN
END

/* Find the HPFS386.INI file.  Default is bootdrive:\IBM386FS\HPFS386.INI     */
/* But it could be overridden in the IFS= line in CONFIG.SYS.                 */

configsys = bootdrive || "\CONFIG.SYS"

line = LINEIN(configsys)
DO WHILE (STREAM(configsys, "S") = "READY" & (POS("HPFS386", line) = 0 | TRANSLATE(SUBWORD(line, 1, 1)) = "REM"))
   line = LINEIN(configsys)
END

options = ""
IF STREAM(configsys, "S") = "READY" THEN DO
   PARSE VAR line WITH "IFS" "=" . options

   /* The first parameter that does not start with a "/" is the override for  */
   /* the .INI file name.                                                     */

   DO WHILE (options <> "" & SUBSTR(options, 1, 1) = "/")
      options  = SUBWORD(options, 2)
   END

   /* If we did not find a non-slash-prefixed parameter, then use the default */
   /* .INI file name and location.                                            */

   IF options = "" THEN
      orgini = bootdrive || "\IBM386FS\HPFS386.INI"

   /* Else, the parameter can be the .INI file name or the path of the .INI   */
   /* file (in which case its name is "HPFS386.INI").  The parameter may      */
   /* contain a drive and or path.  (If it contains a drive, it had better be */
   /* the boot drive.)                                                        */

   ELSE DO
      orgini = SUBWORD(options, 1, 1)   /* Strip off extra parameters.        */

      /* If the name does not have a drive letter, add the boot drive.        */

      IF SUBSTR(orgini, 2, 1) <> ":" THEN DO

         /* If the name is relative, then make it relative to \IBM386FS.      */

         IF SUBSTR(orgini, 1, 1) <> "\" THEN DO
            orgini = "\IBM386FS\" || orgini
         END
         orgini = bootdrive || orgini
      END

      /* Check if the name given is a file that exists. */

      rc = SysFileTree(orgini, file, "F")
      IF file.0 = 0 THEN DO

         /* The file does not exist.  Check if the name given is a path.      */

         rc = SysFileTree(orgini, file, "D")
         IF file.0 <> 0 THEN DO

            /* The name given is a path to the file.  Update our variable to  */
            /* include the file name.  If it exists, we will use it.  If not, */
            /* it will be created.                                            */

            orgini = orgini || "\HPFS386.INI"
         END
      END
   END
END

/* We hit the end of CONFIG.SYS without finding an IFS= line to load HPFS386. */
/* Display a cryptic message that is supposed to indicate that the system is  */
/* not configured for HPFS386.                                                */

ELSE DO
   SAY bootdrive || "\IBM386FS\HPFS386.IFS"
   SAY SysGetMessage(2)                 /* "File not found."                  */
   RETURN
END

/* Get a temporary file for creating a new HPFS386.INI file.  We will create  */
/* it in the same directory as the original .INI file.                        */

workfile = FILESPEC("D", orgini) || FILESPEC("P", orgini) || "HPFS????.INI"
workfile = SysTempFileName(workfile)    /* Generate a temporary file name.    */

/* Search HPFS386.INI for a [COMMIT_HEURISTICS] line.  Copy lines to the      */
/* work file while we go.                                                     */

line = LINEIN(orgini)
DO WHILE (STREAM(orgini, "S") = "READY" & TRANSLATE(SUBWORD(line, 1)) <> "[COMMIT_HEURISTICS]")
   rc = LINEOUT(workfile, line)
   line = LINEIN(orgini)
END

/* Add the requested options to the work file. */

IF PerfSet = nomsg THEN setting = "NO"
ELSE setting = "YES"

rc = LINEOUT(workfile, "[commit_heuristics]")
rc = LINEOUT(workfile, "async = " || setting)
rc = LINEOUT(workfile, "sort  = " || setting)
rc = LINEOUT(workfile, "merge = " || setting)
rc = LINEOUT(workfile, "")

/* Skip over the rest of the [COMMIT_HEURISTICS] section, if it exists. */

IF STREAM(orgini, "S") = "READY" THEN DO
   line = LINEIN(orgini)
   DO WHILE (STREAM(orgini, "S") = "READY" & SUBSTR(SUBWORD(line, 1), 1, 1) <> "[")
      IF SUBSTR(SUBWORD(line, 1), 1, 1) = ";" THEN
         rc = LINEOUT(workfile, line)   /* Copy comment lines. */
      line = LINEIN(orgini)
   END
END

/* Copy the rest of the original file to the work file. */

DO WHILE (STREAM(orgini, "S") = "READY")
   rc = LINEOUT(workfile, line)
   line = LINEIN(orgini)
END

/* Close the files. */

rc = STREAM(orgini, "C", "CLOSE")
rc = STREAM(workfile, "C", "CLOSE")

/* Backup the original HPFS386.INI file to filename.xxx, where xxx is the     */
/* first available number, 001, 002, 003, etc., that is available.            */

i = 1
extoffset = POS(".", orgini)            /* Find the offset of the file        */
                                        /* extension.                         */
IF extoffset = 0 THEN DO                /* If the file does not have an       */
   backupname = orgini || "." || RIGHT(i, 3, "0")
   extoffset = POS(".", orgini)         /* extension, make one at the end.    */
END
ELSE DO
   backupname = SUBSTR(orgini, 1, extoffset) || RIGHT(i, 3, "0")
END

rc = SysFileTree(backupname, file, "B") /* Check if this file name is in use. */
DO WHILE file.0 <> 0                    /* Loop trying other names until a    */
   i = i + 1                            /* free one is found.                 */
   backupname = SUBSTR(backupname, 1, extoffset) || RIGHT(i, 3, "0")
   rc = SysFileTree(backupname, file, "B")
END

/* Rename the original HPFS386.INI, if it exists, to the backup name.  The    */
/* RENAME command takes only the file name for its second parameter.  It will */
/* not accept a fully qualified path for the target, even if the path is to   */
/* the same drive and path as the source name.  Therefore we must strip the   */
/* path from the file name for the target.                                    */

rc = SysFileTree(orgini, file, "F")
IF file.0 <> 0 THEN DO
   "rename" orgini FILESPEC("N", backupname)
END

/* Rename the work file to the original HPFS386.INI file name.  Same rules    */
/* about RENAME apply here.                                                   */

"rename" workfile FILESPEC("name", orgini)

/* Say we're done.  Tell the user about the backup file, if it was created.   */

IF file.0 <> 0 THEN DO
   SAY SysGetMessage(3853, "NET.MSG", orgini, backupname)
                                        /* "Backed up file %1 to %2."         */
END
SAY SysGetMessage(3500, "NET.MSG")   /* "The command completed successfully." */

RETURN
